module ES where

import Data.List
import Equation
import Term

type ES = [Equation]

-- pretty printer.
showES :: ES -> String
showES es = unlines [ showEquation e | e <- es ]

-- If you want to distinguish l = r from r = l,
-- you cannot use `union`. Use ++ instead.
union :: ES -> ES -> ES
union es1 es2 = nubBy Equation.variant (es1 ++ es2)

concat :: [ES] -> ES
concat []           = []
concat (es1 : rest) = ES.union es1 (ES.concat rest)

elemEq :: Equation -> ES -> Bool
elemEq e es = any (Equation.variant e) es

-- es1 \ es2
diff :: ES -> ES -> ES
diff es1 es2 = [ e | e <- es1, not (ES.elemEq e es2) ]

-- E^{-1}
inverse :: ES -> ES
inverse es = [ (r, l) | (l, r) <- es]

sortByWeight :: ES -> ES
sortByWeight es = map snd (sortBy (\(d1, _) (d2, _) -> compare d1 d2)
                                  [(sz, e) | e@(s,t) <- es, let sz = Term.size s + Term.size t])

varsInES :: ES -> [String]
varsInES ps = nub [x | p <- ps, x <- varsInPair p]

functionsInES :: ES -> [String]
functionsInES ps = nub [f | p <- ps, f <- functionsInPair p]
