module LPO where
-- Lexicographic Path Order

import Data.List

import Term
import TRS

-- [a,b,c] means a > b > c
type Precedence = [String]

showPrec :: [String] -> String
showPrec prec = intercalate " > " prec 

-- `gtPrec > f g` tests `f > g`
gtPrec :: Precedence -> String -> String -> Bool
gtPrec [] f g = error (f ++ " and " ++ g ++ " is not found.")
gtPrec (h : hs) f g
  | f == g = False
  | h == f = True
  | h == g = False
  | otherwise = gtPrec hs f g

-- `gtLPO > s t` tests `s >_lpo t`.
gtLPO :: Precedence -> Term -> Term -> Bool
gtLPO _ (V _) _ = False
gtLPO _ s (V x) = x `elem` (variables s)
gtLPO prec s@(F f ss) t@(F g ts)
  | any (\si -> si == t || gtLPO prec si t) ss = True
  | gtPrec prec f g && isGtLPOThanAny s ts = True
  | f == g && isGtLPOThanAny s ts && lexGtLPO ss ts = True
  | otherwise = False
  where isGtLPOThanAny s ts = all (\tj -> gtLPO prec s tj) ts
        lexGtLPO [] [] = False
        lexGtLPO (s : ss) (t : ts)
          | s == t = lexGtLPO ss ts
          | gtLPO prec s t = True
          | otherwise = False

-- l >_lpo r for all rules l -> r in the TRS
lpoTerminating :: Precedence -> TRS -> Bool
lpoTerminating pre trs = all (\(l,r) -> gtLPO pre l r) trs
