module LPOAuto where

import Data.List

import Term
import TRS
import SMT

type LPODic = [((Term, Term), String)]

unsafeLookup :: (Eq a) => a -> [(a, b)] -> b
unsafeLookup a abs
  | Just b <- lookup a abs = b
  | otherwise = undefined

encodeLPO2aDic :: LPODic -> Term -> Term -> Formula
encodeLPO2aDic dic (F f ss) t@(F _ _) =
  case [True | si <- ss, si == t] of
    _ : _ -> trueFormula
    [] -> orFormula  [ BoolVar var | si <- ss,
                                     let var = unsafeLookup (si, t) dic ]
encodeLPO2aDic _ _ _ = undefined

encodeLPO2bDic :: LPODic -> Term -> Term -> Formula
encodeLPO2bDic dic s@(F f _) (F g ts)  | f /= g = andFormula (p1 : ps)
  | otherwise = falseFormula
  where p1 = Gt (Var f) (Var g)
        ps = [ BoolVar var | tj <- ts,
                             let var = unsafeLookup (s, tj) dic]
encodeLPO2bDic _ _ _ = undefined

-- NOTE: LPO2b does not subsume LPO2c.
-- consider f(0, x) >_lpo 0 with empty precedence.
encodeLPO2cDic :: LPODic -> Term -> Term -> Formula
encodeLPO2cDic dic s@(F f ss) (F g ts)
  | f == g = andFormula ((lex ss ts) : ps)
  | otherwise = falseFormula
  where ps = [ BoolVar var | tj <- ts, let var = unsafeLookup (s, tj) dic]
        lex _ [] = falseFormula 
        lex [] _ = falseFormula
        lex (si : ss) (tj : ts)
          | si == tj = lex ss ts
          | otherwise = BoolVar (unsafeLookup (si, tj) dic)
encodeLPO2cDic _ _ _ = undefined

-- cf. term rewriting and all that, Definition 5.4.12
-- encode a rule l -> r into smt-lib constraints for l >_lpo r 
-- encodeLPO l r
encodeLPODic :: LPODic -> Term -> Term -> Formula
encodeLPODic _ (V _) _ = falseFormula
encodeLPODic _ s (V x)
  | x `elem` (variables s) = trueFormula
  | otherwise = falseFormula
encodeLPODic dic s t = orFormula [encodeLPO2aDic dic s t, encodeLPO2bDic dic s t, encodeLPO2cDic dic s t]

constructDic :: TRS -> LPODic
constructDic rules = zip possiblePairs [ "lpo" ++ show i | i <- [0..]]
  where possiblePairs = nub [ (s, t) | (l, r) <- rules, s <- subterms l, t <- subterms r]

compareBySMTValue (_, n1) (_, n2) = compare (smtValueUnsafeCastInt n2) (smtValueUnsafeCastInt n1)
