import Equation
import ES
import TPTP
import Horn
import Prover
import System.Environment

data Mode = Prover String | Help

convertEquation :: Equation -> TPTPInput
convertEquation (l, r) = CNF "some_equation" "axiom" [(True, Equation l r)]

convertGoal :: Equation -> TPTPInput
convertGoal (l, r) = CNF "some_equation" "negated_conjecture" [(False, Equation l r)]

convertES :: ES -> TPTP
convertES es = [ convertEquation e | e <- es ]

parseArgs :: [String] -> Mode
parseArgs ["-h"] = Help
parseArgs ["--help"] = Help
parseArgs [fname] = Prover fname
parseArgs _ = Help

dispatch :: Mode -> IO ()
dispatch Help = do
  putStrLn "Usage: tofu <file>"
  putStrLn "<file> is in TPTP format."
dispatch (Prover fname) = do
  result <- readTPTP fname
  case result of
    Left e -> do
      putStrLn "ERROR"
      print e
    Right tptp -> do
      case splitIf tptp of
        Nothing      -> do
          putStrLn "ERROR"
          putStrLn "\n-- Input:"
          putStr (showTPTP tptp)
          putStrLn "\n-- Note:"
          putStrLn "The input contains a non-Horn-clause."
        Just (es, e@(s, t)) -> do
          putStrLn "\n-- Input:"
          putStr (showTPTP tptp)
          putStrLn "\n-- Axioms:"
          putStr (showTPTP (convertES es ++ [convertGoal e]))
          if s == t -- trivial model found
            then putStrLn "% SZS status Satisfiable"
            else do res <- prover es
                    if res
                      then putStrLn "% SZS status Unsatisfiable"
                      else putStrLn "% SZS status Satisfiable"

main :: IO ()
main = do
  args <- getArgs
  dispatch (parseArgs args)
