module Rule where

import Data.List
import Term
import Substitution

type Rule  = (Term, Term)

-- pretty printer.

showRule :: Rule -> String
showRule (l, r) = show l ++ " -> " ++ show r

-- cf. Term Rewriting and All That, Definition 4.2.1
-- (l, r) is rewrite rule
-- if l is not variable and Var(r) \subseteq Var(l)
isValidRule :: Rule -> Bool
isValidRule (V _, _) = False
isValidRule (l, r) =
  case [ rvar | rvar <- variables r, not (rvar `elem` (variables l))] of
    [] -> True
    _ -> False

-- l1 -> r1 and l2 -> r2 are the same rules
-- if (l1 -> r1) \preceq (l2 -> r2) and (l2 -> r2) \preceq (l1 -> r1)
-- where (l1 -> r1) \preceq (l2 -> r2)
--       if there exists substitution σ such that l1 σ = l2 and r1 σ = r2
variant :: Rule -> Rule -> Bool
variant rule1 rule2 = compare rule1 rule2 && compare rule2 rule1
  where compare (l1, r1) (l2, r2) =
          case match l1 l2 of -- NOTE: we can't use unify. consider f(x,x) -> x and f(x,y) -> x.
            Nothing -> False
            Just sigma -> substitute r1 sigma == r2

rename :: Rule -> Int -> (Int, Rule)
rename (l,r) n = (n + length vars, (substitute l sbst, substitute r sbst))
                 where vars = nub $ variables l
                       sbst = zip vars $ map (\(v,n) -> V $ v ++ show n) (zip vars [n..])
