module Term where

import Data.List

data Term = V String
          | F String [Term]
          deriving Eq

type TermPair = (Term, Term)

-- smart constructor
constant :: String -> Term
constant f = F f []

size :: Term -> Int
size (V _) = 1
size (F _ ts) = 1 + sum [ size t | t <- ts ]

type Position = [Int]

emptyPosition :: Position
emptyPosition = []

peanoNatToInt :: Term -> Maybe Int
peanoNatToInt (F "0" []) = Just 0
peanoNatToInt (F "s" [t]) = do n <- peanoNatToInt t
                               return (n+1)
peanoNatToInt _ = Nothing

showList' :: Term -> Maybe String
showList' (F "cons" [car, (F "nil" [])]) = Just $ show car
showList' (F "cons" [car, cdr]) = do cdrStr <- showList' cdr
                                     return $ show car ++ "," ++ cdrStr 
showList' _ = Nothing

instance Show Term where
  show (V x)    = x
  show (F "0" []) = "0"
  show p@(F "s" [t]) = case peanoNatToInt p of
                         Just n -> show n
                         Nothing -> "s(" ++ show t ++ ")"
  show (F "nil" []) = "[]"
  show l@(F "cons" [car, cdr]) = case showList' l of
                                     Just s -> "[" ++ s ++ "]"
                                     Nothing -> "cons(" ++ show car ++ "," ++ show cdr ++ ")"
  show (F c []) = c
  show (F f ts) = f ++ "(" ++ intercalate "," [ show t | t <- ts ] ++ ")"

-- Var(t)
variables :: Term -> [String]
variables (V x)    = [x]
variables (F _f ts) = nub [ x | ti <- ts, x <- variables ti ]

varsInPair :: TermPair -> [String]
varsInPair (t, s) = nub (variables t ++ variables s)

functions :: Term -> [String]
functions (V _) = []
functions (F f ts) = nub (f : [f' | t <- ts, f' <- functions t])

functionsInPair :: TermPair -> [String]
functionsInPair (t, s) = nub (functions t ++ functions s)

-- Pos(t)
positions :: Term -> [Position]
positions (V _x) = [[]]
positions (F _f ts) = ([]: [ i : p | (i, t) <- zip [0..] ts, p <- positions t])

functionPositions :: Term -> [Position]
functionPositions (V _x) = []
functionPositions (F _f ts) = ([]: [ i : p | (i, t) <- zip [0..] ts, p <- functionPositions t])

-- subtermAt t p = t|_p
subtermAt :: Term -> Position -> Term
subtermAt t [] = t
subtermAt (V _) _ = undefined
subtermAt (F _f ts) (i : p)
  | i < length ts = subtermAt (ts !! i) p
  | otherwise = undefined

subterms :: Term -> [Term]
subterms t = [subtermAt t p | p <- positions t]

strictSubterms :: Term -> [Term]
strictSubterms t = [subtermAt t p | p <- positions t, p /= []]

isSubtermOf :: Term -> Term -> Bool
isSubtermOf t1 t2 = t1 `elem` (subterms t2)

isStrictSubtermOf :: Term -> Term -> Bool
isStrictSubtermOf t1 t2 = t1 `elem` (strictSubterms t2)

-- replace t u p = t[u]_p
replace :: Term -> Term -> Position -> Term
replace _t u [] = u
replace (V _) _ _ = undefined
replace (F f ts) u (i : p)
  | i < length ts = F f ts'
  | otherwise = undefined
      where ts' = [ if i == j then (replace tj u p) else tj | (j, tj) <- zip [0..] ts]
